C $Header: /u/gcmpack/MITgcm/pkg/dic/dic_biotic_forcing.F,v 1.22 2008/04/08 16:18:43 dfer Exp $
C $Name:  $

#include "DIC_OPTIONS.h"

CBOP
C !ROUTINE: DIC_BIOTIC_FORCING

C !INTERFACE: ==========================================================
      SUBROUTINE DIC_BIOTIC_FORCING( PTR_DIC, PTR_ALK, PTR_PO4,
     &                            PTR_DOP, 
#ifdef ALLOW_O2
     &                            PTR_O2, 
#endif
#ifdef ALLOW_FE
     &                            PTR_FE,
#endif
     &                            bi,bj,imin,imax,jmin,jmax,
     &                             myIter,myTime,myThid)

C !DESCRIPTION:
C updates all the tracers for the effects of air-sea exchange, biological
c activity and remineralization

C !USES: ===============================================================
      IMPLICIT NONE
#include "SIZE.h"
#include "DYNVARS.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "GRID.h"
#include "DIC_VARS.h"

C !INPUT PARAMETERS: ===================================================
C  myThid               :: thread number
C  myIter               :: current timestep
C  myTime               :: current time
C  PTR_DIC              :: dissolced inorganic carbon
C  PTR_ALK              :: alkalinity
C  PTR_PO4              :: phosphate
c  PTR_DOP              :: dissolve organic phosphurous
c  PTR_O2               :: oxygen
C  PTR_FE               :: iron
      INTEGER myIter
      _RL myTime
      INTEGER myThid
      _RL  PTR_DIC(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  PTR_ALK(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  PTR_PO4(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  PTR_DOP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#ifdef ALLOW_O2
      _RL  PTR_O2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#endif
#ifdef ALLOW_FE
      _RL  PTR_FE(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#endif
      INTEGER bi, bj, imin, imax, jmin, jmax

#ifdef ALLOW_PTRACERS
#ifdef DIC_BIOTIC

C !LOCAL VARIABLES: ====================================================
C  i,j,k                  :: loop indices
C  G*                     :: tendency term for the tracers
C  SURA                   :: tendency of alkalinity due to freshwater
C  SURC                   :: tendency of DIC due to air-sea exchange
C                            and virtual flux
C  SURO                   :: tendency of O2 due to air-sea exchange
C  GPO4                   :: tendency of PO4 due to biological productivity,
C                            exchange with DOP pool and reminerization
C  CAR                    :: carbonate changes due to biological 
C                             productivity and remineralization
C  BIOac                  :: biological productivity
C  RDOP                   :: DOP sink due to remineralization
C  pflux                  :: changes to PO4 due to flux and remineralization
C  CAR_S                  :: carbonate sink 
C  cflux                  :: carbonate changes due to flux and remineralization
C  freefe                 :: iron not bound to ligand
      _RL  GDIC(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  GALK(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  GPO4(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  GDOP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  SURA(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL  SURC(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL  SURO(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL  CAR(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  BIOac(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  RDOP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  pflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
CAPHAM
      _RL  FEremin(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hydrodFe(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
CAPHAM
      _RL  exportflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  CAR_S(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  cflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
CTAKA
      _RL  botflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  botfeflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
CTAKA
#ifdef ALLOW_O2
      _RL  GO2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#endif
#ifdef ALLOW_FE
      _RL  GFE(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  freefe(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  feL1(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  feL2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ligand1(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ligand2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironbio(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironrem(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironreb(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironsed(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Irondust(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL  Ironscav(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironsci(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironsco(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  tmpFe(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ironloss(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr) 
#endif
       INTEGER I,J,k
       INTEGER nCALCITEstep
CAPHAM
      _RL  Kscstar(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Kscavin(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Febar(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      _RL  R_sed(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  Ratio_FeP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  FEremin_inorg(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  WFE_inorg(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)

CAPHAM

CEOP

       DO k=1,Nr
         DO j=1-OLy,sNy+OLy
          DO i=1-OLx,sNx+OLx
           RDOP(i,j,k) =0. _d 0
           GDIC(i,j,k) =0. _d 0
           GALK(i,j,k) =0. _d 0
           GPO4(i,j,k) =0. _d 0
           GDOP(i,j,k) =0. _d 0
           CAR(i,j,k)  =0. _d 0
           BIOac(i,j,k)   =0. _d 0
           pflux(i,j,k)   =0. _d 0
           exportflux(i,j,k)=0. _d 0
CTAKA
           botflux(i,j,k)      =0. _d 0
           botfeflux(i,j,k)    =0. _d 0
           FEremin_inorg(i,j,k)=0. _d 0
           WFE_inorg(i,j,k)    =0. _d 0
           Kscavin(i,j,k)      =0. _d 0
           Febar(i,j)      =0. _d 0
           R_sed(i,j,k)      =0. _d 0
           Ratio_FeP(i,j,k)      =3.40 _d -4
CTAKA
           cflux(i,j,k)   =0. _d 0
           CAR_S(i,j,k)   =0. _d 0
#ifdef ALLOW_O2
           GO2(i,j,k)     =0. _d 0
#endif
#ifdef ALLOW_FE
           GFE(i,j,k)     =0. _d 0
           freefe(i,j,k)  =0. _d 0
           tmpFe(i,j,k) = PTR_FE(i,j,k)
#endif
          ENDDO
         ENDDO
       ENDDO
         DO j=1-OLy,sNy+OLy
          DO i=1-OLx,sNx+OLx
           SURA(i,j)   =0. _d 0
           SURC(i,j)   =0. _d 0
           SURO(i,j)   =0. _d 0
          ENDDO
         ENDDO

c carbon air-sea interaction
       CALL DIC_SURFFORCING( PTR_DIC, PTR_ALK, PTR_PO4, SURC,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)

c alkalinity air-sea interaction
       CALL ALK_SURFFORCING( PTR_ALK, SURA,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)

#ifdef ALLOW_O2
c oxygen air-sea interaction
       CALL O2_SURFFORCING( PTR_O2, SURO,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)
#endif

c biological activity
       CALL BIO_EXPORT( PTR_PO4 , 
#ifdef ALLOW_FE
     I           PTR_FE, 
#endif 
     I           BIOac,
     I           bi,bj,imin,imax,jmin,jmax,
     I           myIter,myTime,myThid)

#ifdef ALLOW_FE
             call dic_calc_ligand(PTR_DOP, PTR_O2, PTR_FE,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)       
c find free iron
CAPHAM       call fe_chem(bi,bj,iMin,iMax,jMin,jMax, PTR_FE, freefe,
CAPHAM     &                myIter, mythid)
       call fe_chem(bi,bj,iMin,iMax,jMin,jMax, PTR_FE, freefe,
     &                myIter, mythid)
#endif

CAPHAM
      DO k=1,Nr
         DO j=1-OLy,sNy+OLy
          DO i=1-OLx,sNx+OLx
             Ironloss(i,j,k) = -(tmpFe(i,j,k) - PTR_FE(i,j,k))*
     & ((dTtracerLev(k))**(-1))
          ENDDO
         ENDDO
      ENDDO 
CAPHAM

c flux of po4 from layers with biological activity
       CALL PHOS_FLUX( BIOac, pflux, exportflux,
CTAKA
C     &                    FEremin, botflux,
     &                    botflux,
CTAKA
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)

       CALL fe_remin(Ratio_FeP, BIOac, pflux,
     &                    exportflux,
CTAKA
     &                    FEremin, botflux, freefe,
     &                    Kscstar, 
CTAKA
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)

       CALL fe_remin_inorg(freefe, Kscavin, FEremin_inorg, 
     &           WFE_inorg, 
     I           bi,bj,imin,imax,jmin,jmax,
     I           myIter,myTime,myThid)
CPHAM ------------------------------    

C- Carbonate sink
       DO k=1,Nr
         DO j=jmin,jmax
          DO i=imin,imax
             CAR_S(i,j,k)=BIOac(i,j,k)*R_CP*rain_ratio(i,j,bi,bj)*
     &                    (1. _d 0-DOPfraction) 
          ENDDO
         ENDDO
       ENDDO

c carbonate
#ifdef CAR_DISS
c dissolution only below saturation horizon
c code following method by Karsten Friis
         nCALCITEstep = 3600
         IF(myIter .lt. (nIter0+5) .or.
     &               mod(myIter,nCALCITEstep) .eq. 0)THEN
           CALL CALCITE_SATURATION(PTR_DIC, PTR_ALK, PTR_PO4,
     I                    bi,bj,imin,imax,jmin,jmax,
     I                    myIter,myTime,myThid)
        ENDIF
c
        CALL CAR_FLUX_OMEGA_TOP( BIOac, cflux,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)
#else
c old OCMIP way
        CALL CAR_FLUX( CAR_S, cflux,
     &                    bi,bj,imin,imax,jmin,jmax,
     &                    myIter,myTime,myThid)
#endif
        DO j=jmin,jmax
          DO i=imin,imax
            DO k=1,9
            Febar(i,j) = Febar(i,j)+PTR_Fe(i,j,k)*drC(k)
     &     *maskC(i,j,k,bi,bj)
             ENDDO
         ENDDO
       ENDDO

c add all tendencies for PO4, DOP, ALK, DIC
       DO k=1,Nr
         DO j=jmin,jmax
          DO i=imin,imax
#ifdef DIC_NO_NEG
           RDOP(i,j,k)= MAX(maskC(i,j,k,bi,bj)*KDOPRemin*PTR_DOP(i,j,k)
     &                     ,0. _d 0)
#else
           RDOP(i,j,k)= maskC(i,j,k,bi,bj)*KDOPRemin*PTR_DOP(i,j,k)
#endif
           GPO4(i,j,k)=-BIOac(i,j,k)+pflux(i,j,k) + RDOP(i,j,k)
           car(i,j,k) = cflux(i,j,k) - CAR_S(i,j,k)
           GDOP(i,j,k)=+BIOac(i,j,k)*DOPfraction - RDOP(i,j,k)
           GALK(i,j,k)=+2. _d 0 *car(i,j,k)-R_NP*GPO4(i,j,k)
           GDIC(i,j,k)=car(i,j,k)+R_CP*GPO4(i,j,k)

#ifdef ALLOW_O2
           if (PTR_O2(i,j,k).GT.O2crit) then
             GO2(i,j,k)= R_OP*GPO4(i,j,k)
           else
             GO2(i,j,k)= 0. _d 0
           endif
#endif
#ifdef ALLOW_FE
           FeL1(i,j,k) = FeL(i,j,k,bi,bj,1)
           FeL2(i,j,k) = FeL(i,j,k,bi,bj,2)
           Ligand1(i,j,k) = Ligand(i,j,k,bi,bj,1)
           Ligand2(i,j,k) = Ligand(i,j,k,bi,bj,2)

CAPHAM luxury FeP ratio uptake based on Fe concentration (Ellery et al)
           if (j.LE.3) then
           Ratio_FeP(i,j,k) = 
     &     MAX(3.40 _d -4,
     &     0.0015*Febar(i,j)/(5.6 _d -5 
     &     + Febar(i,j)))

           else if (j.GE.7
     &    .AND.i.GE.15 
     &    .AND.i.LE.37) then
    
           Ratio_FeP(i,j,k) =
     &     MAX(3.40 _d -4,
     &     0.0015*Febar(i,j)/(5.6 _d -5
     &     + Febar(i,j)))
                     
           else
           Ratio_FeP(i,j,k)=3.40 _d -4
           endif

            
CAPHAM Turn off inorganic scavenging when O2 concentration is lower than
C a threshold
 
           If (PTR_O2(i,j,k).GT.0.03 _d 0) then           
           GFE(i,j,k) =-BIOac(i,j,k)*Ratio_FeP(i,j,k)+ FEremin(i,j,k)
     &                  +RDOP(i,j,k)*Ratio_FeP(i,j,k) 
     &                  -Kscstar(i,j,k)*freefe(i,j,k)
     &                  -Kscavin(i,j,k)*freefe(i,j,k)
     &                  +FEremin_inorg(i,j,k)
     &                  +hydroFe(i,j,k,bi,bj)*maskC(i,j,k,bi,bj)
           else

           Kscavin(i,j,k) = 0.0 _d 0
           FEremin_inorg(i,j,k) = 0.0 _d 0

           GFE(i,j,k) =-BIOac(i,j,k)*Ratio_FeP(i,j,k)+ FEremin(i,j,k)
     &                  +RDOP(i,j,k)*Ratio_FeP(i,j,k)
     &                  -Kscstar(i,j,k)*freefe(i,j,k)
     &                  -Kscavin(i,j,k)*freefe(i,j,k)
     &                  +FEremin_inorg(i,j,k)
     &                  +hydroFe(i,j,k,bi,bj)*maskC(i,j,k,bi,bj) 
           endif

            Ironscav(i,j,k)=-(Kscavin(i,j,k) + Kscstar(i,j,k))
     &                      *freefe(i,j,k)
     &      *maskC(i,j,k,bi,bj)

            Ironsci(i,j,k) = -Kscavin(i,j,k)*freefe(i,j,k) 
     &      *maskC(i,j,k,bi,bj)
            
            Ironsco(i,j,k) = -Kscstar(i,j,k)*freefe(i,j,k)
     &      *maskC(i,j,k,bi,bj)   
            
            Ironbio(i,j,k) = -BIOac(i,j,k)*Ratio_FeP(i,j,k)
     &      *maskC(i,j,k,bi,bj)

            Ironrem(i,j,k) = (Ratio_FeP(i,j,k)*RDOP(i,j,k)
     &      +FEremin(i,j,k))*maskC(i,j,k,bi,bj)

            Ironreb(i,j,k) = (Ratio_FeP(i,j,k)*(pflux(i,j,k)
     &      + RDOP(i,j,k)))*maskC(i,j,k,bi,bj)

            HydrodFe(i,j,k)=hydroFe(i,j,k,bi,bj)*maskC(i,j,k,bi,bj)

CTAKA add shelf iron source
CAPHAM Shelf iron source based on the oxygen concentration

C           If (PTR_O2(i,j,k).GT.0.03 _d 0) then
             R_sed(i,j,k) = R_sed_const
C           else
C             R_sed(i,j,k) = R_sed_const*5.0
C           endif

C           IF ( k .EQ. Nr ) THEN
C               botfeflux(i,j,k)=botflux(i,j,k-1)*R_CP*R_sed(i,j,k)
C           ELSEIF (maskC(i,j,k,bi,bj) .EQ. 1. .AND. 
C     &          maskC(i,j,k+1,bi,bj) .EQ. 0. ) THEN
C               botfeflux(i,j,k)=botflux(i,j,k-1)*R_CP*R_sed(i,j,k)
C           ELSE
               botfeflux(i,j,k)=0.0
C           ENDIF

CAPHAM offline sediment flux

           IF ( k .EQ. Nr ) THEN
               botfeflux(i,j,k)=sedf(i,j,k,bi,bj)
          ELSEIF (maskC(i,j,k,bi,bj) .EQ. 1. .AND.
     &          maskC(i,j,k+1,bi,bj) .EQ. 0. ) THEN
               botfeflux(i,j,k)=sedf(i,j,k,bi,bj)
           ELSE
               botfeflux(i,j,k)=0.0
           ENDIF



           GFE(i,j,k)=GFE(i,j,k)+botfeflux(i,j,k)
     &             *recip_drF(k)*recip_hFacC(i,j,k,bi,bj)
     &             *maskC(i,j,k,bi,bj)

           Ironsed(i,j,k)= botfeflux(i,j,k)
     &             *recip_drF(k)*recip_hFacC(i,j,k,bi,bj)
     &             *maskC(i,j,k,bi,bj)

CTAKA
#endif
          ENDDO
         ENDDO
       ENDDO

         DO j=jmin,jmax
          DO i=imin,imax
               GALK(i,j,1)=GALK(i,j,1)+SURA(i,j)
               GDIC(i,j,1)=GDIC(i,j,1)+SURC(i,j)
#ifdef ALLOW_O2
               GO2(i,j,1) =GO2(i,j,1)+SURO(i,j)
#endif
#ifdef ALLOW_FE
               GFE(i,j,1)=GFE(i,j,1)+alpfe*
     &                    InputFe(i,j,bi,bj)*recip_drF(1)
     &                       *recip_hFacC(i,j,1,bi,bj)

               Irondust(i,j)= alpfe*
     &                    InputFe(i,j,bi,bj)*recip_drF(1)
     &                       *recip_hFacC(i,j,1,bi,bj)

#endif
          ENDDO
         ENDDO

C update
       DO k=1,Nr
         DO j=jmin,jmax
          DO i=imin,imax
           PTR_DIC(i,j,k)=
     &      PTR_DIC(i,j,k)+GDIC(i,j,k)*dTtracerLev(k)
           PTR_ALK(i,j,k)=
     &      PTR_ALK(i,j,k)+GALK(i,j,k)*dTtracerLev(k)
           PTR_PO4(i,j,k)=
     &      PTR_PO4(i,j,k)+GPO4(i,j,k)*dTtracerLev(k)
           PTR_DOP(i,j,k)=
     &      PTR_DOP(i,j,k)+GDOP(i,j,k)*dTtracerLev(k)
#ifdef ALLOW_O2
           PTR_O2(i,j,k)=
     &      PTR_O2(i,j,k)+GO2(i,j,k)*dTtracerLev(k)
#endif
#ifdef ALLOW_FE
           PTR_FE(i,j,k)=
     &      PTR_FE(i,j,k)+GFE(i,j,k)*dTtracerLev(k)
#endif
          ENDDO
         ENDDO
       ENDDO

#ifdef ALLOW_FE
#ifdef MINFE
c find free iron and get rid of insoluble part
Ctaka       call fe_chem(bi,bj,iMin,iMax,jMin,jMax, PTR_FE, freefe,
Ctaka     &                myIter, mythid)
CTAKA  
CTAKA       IF ( bi .eq. 1 .and. bj .eq. 1) THEN
CTAKA           print *, 'fel1=',FeL(20,20,1,bi,bj,1)
CTAKA       ENDIF
#endif 
#endif


#ifdef ALLOW_TIMEAVE 
c save averages
      IF ( taveFreq.GT.0. ) THEN
      DO k=1,Nr
         DO j=jmin,jmax
          DO i=imin,imax
            BIOave(i,j,k,bi,bj)   =BIOave(i,j,k,bi,bj)+
     &                             BIOac(i,j,k)*deltaTclock
            CARave(i,j,k,bi,bj)   =CARave(i,j,k,bi,bj)+
     &                             CAR(i,j,k)*deltaTclock
            OmegaCave(i,j,k,bi,bj)=OmegaCave(i,j,k,bi,bj)+
     &                             OmegaC(i,j,k,bi,bj)*deltaTclock
            pfluxave(i,j,k,bi,bj) =pfluxave(i,j,k,bi,bj) +
     &                             pflux(i,j,k)*deltaTclock
            epfluxave(i,j,k,bi,bj)=epfluxave(i,j,k,bi,bj) +
     &                             exportflux(i,j,k)*deltaTclock
            cfluxave(i,j,k,bi,bj) =cfluxave(i,j,k,bi,bj) +
     &                             cflux(i,j,k)*deltaTclock
          ENDDO
         ENDDO
      ENDDO
         DO j=jmin,jmax
          DO i=imin,imax
              SURave(i,j,bi,bj)    =SURave(i,j,bi,bj)+
     &                              SURC(i,j)*deltaTclock
#ifdef ALLOW_O2
              SUROave(i,j,bi,bj)   =SUROave(i,j,bi,bj)+
     &                              SURO(i,j)*deltaTclock
#endif
              pCO2ave(i,j,bi,bj)   =pCO2ave(i,j,bi,bj)+
     &                              pCO2(i,j,bi,bj)*deltaTclock
              pHave(i,j,bi,bj)     =pHave(i,j,bi,bj)+
     &                              pH(i,j,bi,bj)*deltaTclock
              fluxCO2ave(i,j,bi,bj)=fluxCO2ave(i,j,bi,bj)+
     &                           fluxCO2(i,j,bi,bj)*deltaTclock
          ENDDO
         ENDDO
      do k=1,Nr
       dic_timeave(bi,bj,k)=dic_timeave(bi,bj,k)+deltaTclock
      enddo
      ENDIF
#endif /* ALLOW_TIMEAVE*/

C---+----1----+----2----+----3----+----4----+----5----+----6----+----7-|--+----|

#ifdef ALLOW_DIAGNOSTICS

      IF ( useDiagnostics ) THEN

        CALL DIAGNOSTICS_FILL(BIOac  ,'DICBIOA ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(CAR    ,'DICCARB ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(pCO2   ,'DICPCO2 ',0,1 ,1,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(fluxCO2,'DICCFLX ',0,1 ,1,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(pH     ,'DICPHAV ',0,1 ,1,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(SURC   ,'DICTFLX ',0,1 ,2,bi,bj,myThid)
#ifdef ALLOW_O2
        CALL DIAGNOSTICS_FILL(SURO   ,'DICOFLX ',0,1 ,2,bi,bj,myThid)
#endif
CAPHAM
        CALL DIAGNOSTICS_FILL(FeL1,   'FeL1    ',0,Nr,2,bi,bj,myThid )
        CALL DIAGNOSTICS_FILL(FeL2,   'FeL2    ',0,Nr,2,bi,bj,myThid )
        CALL DIAGNOSTICS_FILL(Kscstar,'Kscstar ',0,Nr,2,bi,bj,myThid )
        CALL DIAGNOSTICS_FILL(Kscavin,'Kscavin ',0,Nr,2,bi,bj,myThid )
        CALL DIAGNOSTICS_FILL(Ratio_FeP,'Ratio_FeP ',0,Nr,2,
     &  bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Rptl_FeP,'Rptl_FeP',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ligand1, 'Ligand1 ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ligand2, 'Ligand2 ',0,Nr,2,bi,bj,myThid) 
        CALL DIAGNOSTICS_FILL(Ironscav, 'Ironscav ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironsci, 'Ironsci ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironsco, 'Ironsco ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironbio, 'Ironbio ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironrem, 'Ironrem ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironreb, 'Ironreb ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(FEremin_inorg, 'FEremin_inorg ',0,Nr,2,
     &  bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(WFE_inorg, 'WFE_inorg ',0,Nr,2,
     &  bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironsed, 'Ironsed ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(botfeflux, 'botfeflux ',0,Nr,2,bi,bj,
     &  myThid)
         CALL DIAGNOSTICS_FILL(sedf, 'sedf ',0,Nr,2,bi,bj,
     &  myThid)
        CALL DIAGNOSTICS_FILL(Irondust, 'Irondust ',0,1 ,1,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(Ironloss, 'Ironloss ',0,Nr,2,bi,bj,myThid)
        CALL DIAGNOSTICS_FILL(HydrodFe, 'IronHydro ',
     &  0,Nr,2,bi,bj,myThid)
      ENDIF

#endif /* ALLOW_DIAGNOSTICS */

#endif /* DIC_BIOTIC */
#endif /* ALLOW_PTRACERS */

c
       RETURN
       END
