#include "DIC_OPTIONS.h"

CBOP

C !INTERFACE: ==========================================================
      SUBROUTINE fe_remin (Ratio_FeP, BIOac, pflux, exportflux,
     I           FEremin,botflux,freefe,Kscstar,
     I           bi,bj,imin,imax,jmin,jmax,
     I           myIter,myTime,myThid)

C !DESCRIPTION:
C Calculate the PO4 flux to depth from bio activity

C !USES: ===============================================================
      IMPLICIT NONE
#include "SIZE.h"
#include "DYNVARS.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "GRID.h"
#include "DIC_VARS.h"

C !INPUT PARAMETERS: ===================================================
C  myThid               :: thread number
C  myIter               :: current timestep
C  myTime               :: current time
C  BIOac                :: biological productivity
      INTEGER myIter
      _RL myTime
      INTEGER myThid
      _RL  BIOac(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      INTEGER imin, imax, jmin, jmax, bi, bj

C !OUTPUT PARAMETERS: ===================================================
C  pflux                :: changes to PO4 due to flux and reminerlization
      _RL pflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL exportflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
CTAKA
      _RL botflux(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL FEremin(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL Ratio_FeP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)

#ifdef ALLOW_PTRACERS
#ifdef DIC_BIOTIC

C !LOCAL VARIABLES: ====================================================
C  i,j,k                  :: loop indices
c  ko                     :: loop-within-loop index
c  bexport                :: flux of phosphorus from base each "productive"
c                            layer
c  depth_u, depth_l       :: depths of upper and lower interfaces
c  flux_u, flux_l         :: flux through upper and lower interfaces
c  zbase                  :: depth of bottom of current productive layer
       _RL zbase
       _RL bexport(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
       _RL Kscstar(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
       _RL freefe(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
       INTEGER I,J,k, ko, kop1, maskp1
       _RL depth_u, depth_l
       _RL flux_u, flux_l
       _RL flux_l2
CEOP

C- Calculate scavenging of fe from each layer
       DO j=jmin,jmax
        DO i=imin,imax
         DO k=1,Nr

CTAKA if in the water, calculate scavenging rate
           IF (hFacC(i,j,k,bi,bj).gt.0. _d 0) THEN

            IF ( k .lt. Nr ) THEN
             Kscstar(i,j,k) = Kscav*(
     &          (.5*(exportflux(i,j,k)+exportflux(i,j,k+1))*R_CP*12.0)/
     &          (1.85e-4+5.787e-7*MAX(0.0,(-rc(k)-80.0))))**0.58
            ELSE
             Kscstar(i,j,k) = Kscav*(
     &          (exportflux(i,j,k)*R_CP*12.0)/
     &          (1.85e-4+5.787e-7*MAX(0.0,(-rc(k)-80.0))))**0.58
            ENDIF

CTAKA if not in the water, no scavenging
          ELSE
           Kscstar(i,j,k) = 0.0
          ENDIF

C- now calculate the Fe:P ratio in the sinking particle

C- Here, consider a simple case where Rptl_FeP is vertically uniform
C          IF (k .eq. 1) THEN
C             Rptl_FeP(i,j,1,bi,bj) = R_FePmin
C          ELSE
C             Rptl_FeP(i,j,1,bi,bj) = Rptl_FeP(i,j,1,bi,bj)
C- for now, assume a constant euphotic depth, k=9, i.e. 185m depth on
C- the ECCO grid
C     &        + Kscstar(i,j,k)*freefe(i,j,k)*drC(k)
C     &          /(exportflux(i,j,9)+1.E-13) 
C     &          *maskC(i,j,k,bi,bj)
c          ENDIF
C         ENDDO

CTAKA --- scheme for now ---
          IF (k .eq. 1) THEN
C- At the surface, particle ratio is the same as uptake ratio
             Rptl_FeP(i,j,k,bi,bj) = Ratio_FeP(i,j,k)

          ELSE

C - At depth, vertically integrate the particle ratio 
             Rptl_FeP(i,j,k,bi,bj) = Rptl_FeP(i,j,k-1,bi,bj)
     &        + Kscstar(i,j,k)*freefe(i,j,k)
     &        /(exportflux(i,j,k)+1.E-13)*drc(k)
     &        *maskc(i,j,k,bi,bj)                

C - Let Rptl be lower than the maximum value (set to 0.01 for now)
C - this is a safety mechanism to avoid extremely high ratio
            Rptl_FeP(i,j,k,bi,bj)=min(1.0E-2,Rptl_FeP(i,j,k,bi,bj))
          ENDIF

C         DO k=1,Nr
C             Rptl_FeP(i,j,k,bi,bj)= Rptl_FeP(i,j,1,bi,bj)

             FEremin(i,j,k)=pflux(i,j,k)
     &                  *Rptl_FeP(i,j,k,bi,bj)
     &                  *maskc(i,j,k,bi,bj)
         ENDDO
        ENDDO
       ENDDO
c
#endif
#endif
       RETURN
       END
