#include "DIC_OPTIONS.h"

CAPHAM
C !ROUTINE: DIC_CALC_LIGAND

C !INTERFACE: ==========================================================
      SUBROUTINE DIC_CALC_LIGAND( PTR_DOP, PTR_O2,PTR_FE, 
     &                            bi,bj,imin,imax,jmin,jmax,
     &                             myIter,myTime,myThid)

C !DESCRIPTION:
C calculates the organic ligands based on DOP and AOU
C calculates the Fe:P uptake ratio

C !USES: ===============================================================
      IMPLICIT NONE
#include "SIZE.h"
#include "DYNVARS.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "GRID.h"
#include "DIC_VARS.h"

C !INPUT PARAMETERS: ===================================================
C  myThid               :: thread number
C  myIter               :: current timestep
C  myTime               :: current time
c  PTR_DOP              :: dissolve organic phosphurous
c  PTR_O2               :: oxygen
      INTEGER myIter
      _RL myTime
      INTEGER myThid
      _RL  PTR_DOP(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  PTR_O2(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  PTR_FE(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      INTEGER bi, bj, imin, imax, jmin, jmax

#ifdef ALLOW_PTRACERS
#ifdef DIC_BIOTIC

C !LOCAL VARIABLES: ====================================================
C  i,j,k                  :: loop indices
       INTEGER I,J,k
      _RL o2satlocal
      _RL aoulocal
      _RL  aTT
      _RL  aTK
      _RL  aTS
      _RL  aTS2
      _RL  aTS3
      _RL  aTS4
      _RL  aTS5
      _RL  o2s
      _RL  ttemp
      _RL  stemp
      _RL  oCnew
CEOP
       DO k=1,Nr
         DO j=1-OLy,sNy+OLy
          DO i=1-OLx,sNx+OLx
CAPHAM
CAPHAM          Ligand(i,j,k,bi,bj,1)=1.E-6 * exp(RC(k)*1.E-2)
C     &        *maskC(i,j,k,bi,bj)
CAPHAMnoligand           Ligand(i,j,k,bi,bj,2)=(1.E-6 * (1.0-1.0*exp(RC(k)*1.E-2)) )
CAPHAM     &        *maskC(i,j,k,bi,bj)

           IF (maskC(i,j,k,bi,bj).NE.0.) THEN
              ttemp = theta(i,j,k,bi,bj)
              stemp = salt(i,j,k,bi,bj)
C determine saturation O2
C using Garcia and Gordon (1992), L&O (mistake in original ?)
              aTT  = 298.15 _d 0 -ttemp
              aTK  = 273.15 _d 0 +ttemp
              aTS  = log(aTT/aTK)
              aTS2 = aTS*aTS
              aTS3 = aTS2*aTS
              aTS4 = aTS3*aTS
              aTS5 = aTS4*aTS

              oCnew  = oA0 + oA1*aTS + oA2*aTS2 + oA3*aTS3 +
     &            oA4*aTS4 + oA5*aTS5
     &          + stemp*(oB0 + oB1*aTS + oB2*aTS2 + oB3*aTS3)
     &          + oC0*(stemp*stemp)

              o2s = EXP(oCnew)
c Convert from ml/l to mol/m^3
              o2satlocal = o2s/22391.6 _d 0 * 1. _d 3
              aoulocal = o2satlocal - PTR_O2(i,j,k)
CAPHAM set L1 = 0
              ligand(i,j,k,bi,bj,1)= R_L1dop*PTR_DOP(i,j,k)
CAPHAM
C               ligand(i,j,k,bi,bj,1)=0.0
C              ligand(i,j,k,bi,bj,2)=R_L2aou*aoulocal
CAPHAM Parameterize L2 followed Mitsumi et al., 2013
             ligand(i,j,k,bi,bj,2)= K_L2*L2_b + 
     &        (1-K_L2)* R_L2aou*aoulocal

CAPHAM set L2 = 1nM
C              ligand(i,j,k,bi,bj,2)= 1. _d -6
CAPHAM
           ELSE
              ligand(i,j,k,bi,bj,1)=0.0
              ligand(i,j,k,bi,bj,2)=0.0
           ENDIF
          ENDDO
         ENDDO
       ENDDO

#endif /* DIC_BIOTIC */
#endif /* ALLOW_PTRACERS */

c
       RETURN
       END
